<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    INVERTUS, UAB www.invertus.eu <support@invertus.eu>
 * @copyright Copyright (c) permanent, INVERTUS, UAB
 * @license   Addons PrestaShop license limitation
 * @see       /LICENSE
 *
 * International Registered Trademark & Property of INVERTUS, UAB
 */

use Invertus\SwedBank\Config\BankLink;
use Invertus\SwedBank\Service\OrderHandler;
use Invertus\SwedBank\Util\CartValidation;
use Invertus\SwedBank\Util\OrderValidation;
use SwedbankPaymentPortal\BankLink\CommunicationEntity\Type\PaymentMethod;
use SwedbankPaymentPortal\BankLink\CommunicationEntity\Type\ServiceType;
use SwedbankPaymentPortal\BankLink\PurchaseBuilder;
use SwedbankPaymentPortal\SwedbankPaymentPortal;
use SwedbankPaymentPortal\UrlCallback;

class SwedBankBanklinkModuleFrontController extends ModuleFrontController
{
    const FILENAME = 'banklink';
    /**
     * @var SwedBank
     */
    public $module;

    public function checkAccess()
    {
        $module = $this->module;
        $customer = $this->context->customer;

        $isPaymentProcess = Tools::isSubmit('id_order');

        $link = $this->context->link;
        $redirectUrl = $isPaymentProcess ?
            $link->getPageLink('history') :
            $link->getPageLink('order');

        if (!$module->active || !$module->isConfigured()) {
            Tools::redirect($redirectUrl);
        }

        if ($isPaymentProcess) {
            $idOrder = Tools::getValue('id_order');
            $order = new Order($idOrder);
            /** @var OrderValidation $orderValidation */
            $orderValidation = $module->get(OrderValidation::class);
            $isAccessGranted = $orderValidation->validate($order, $customer);
        } else {
            /** @var CartValidation $cartValidation */
            $cartValidation = $module->get(CartValidation::class);
            $isAccessGranted = $cartValidation->validate($this->context->cart, $customer);
        }

        if (!$isAccessGranted) {
            Tools::redirect($redirectUrl);
        }

        return true;
    }

    public function postProcess()
    {
        $moduleName = $this->module->name;
        $link = $this->context->link;
        $paymentMethod = Tools::getValue('payment_method');

        $sppPaymentMethod = $this->getSppPaymentMethod($paymentMethod);
        list($sppServiceType, $sppServiceName) = $this->getServiceType($paymentMethod);

        if (null === $sppServiceType || null === $sppPaymentMethod) {
            Tools::redirect($link->getPageLink('order'));
        }

        /** @var OrderHandler $orderService */
        $orderService = $this->module->get(OrderHandler::class);

        if (Tools::isSubmit('id_order')) {
            $idOrder = Tools::getValue('id_order');
        } else {
            $idOrder =
                $orderService->processOrderCreate($this->context->cart, $this->context->currency, $sppServiceName);
        }

        $order = new Order($idOrder);
        if ($order->payment != $sppServiceName) {
            $order->payment = $sppServiceName;
            $order->save();
        }

        $orderAmountCents = (int) (string) ($order->total_paid_tax_incl * 100);

        $idCurrencyEuro = Currency::getIdByIsoCode('EUR');
        $currencyEuro = new Currency($idCurrencyEuro);

        /** @var SwedbankPaymentPortal $paymentPortal */
        $paymentPortal = $this->module->get(SwedbankPaymentPortal::class);

        $customer = $this->context->customer;

        $urlParams = ['id_order' => $idOrder];
        $confirmUrl = $link->getModuleLink($moduleName, 'banklinkconfirm', $urlParams);

        $languageIso = $orderService->getPaymentLanguage($order->id_lang, $order->id_shop, 'banklink');

        if (false === $languageIso) {
            $languageIso = $this->context->language->iso_code;
        }

        $idUnique = Tools::passwdGen(4);
        SwedbankOrderPayment::associateUniqueId($idOrder, $this->context->shop->id, $idUnique);

        $merchantReference = $orderService->formatMerchantReference($order, $this->context->shop->id);

        $purchaseRequest = (new PurchaseBuilder())
            ->setDescription($merchantReference)
            ->setAmountValue($orderAmountCents)
            ->setAmountExponent(2)
            ->setAmountCurrencyCode($currencyEuro->iso_code_num)
            ->setConsumerEmail($customer->email)
            ->setServiceType($sppServiceType)
            ->setPaymentMethod($sppPaymentMethod)
            ->setSuccessUrl($confirmUrl)
            ->setFailureUrl($confirmUrl)
            ->setMerchantReference($merchantReference)
            ->setLanguage(Tools::strtolower($languageIso))
            ->setPageSetId(1)
            ->getPurchaseRequest();

        $callbackParams = ['id_order' => $order->id, 'secret_key' => Tools::encrypt($moduleName)];
        $callbackUrl = $link->getModuleLink($moduleName, 'banklinkcallback', $callbackParams);

        $transactionHandler = UrlCallback::create($callbackUrl);
        try {
            $response = $paymentPortal->getBankLinkGateway()
                ->initPayment($purchaseRequest, $transactionHandler);

            Tools::redirect($response->getCustomerRedirectUrl());
        } catch (Exception $e) {
            $this->errors[] =
                $this->module->l('Operation failed. Please choose another payment method or try again.', 'banklink');

            $idLang = $this->context->language->id;
            $orderConfirmationUrl = $orderService->getOrderConfirmationUrl($order, $idLang);

            $this->redirectWithNotifications($orderConfirmationUrl);
        }
    }

    /**
     * @param string $paymentMethod
     *
     * @return PaymentMethod
     */
    private function getSppPaymentMethod($paymentMethod)
    {
        switch (Tools::strtoupper($paymentMethod)) {
            case BankLink::PAYMENT_METHOD_SWEDBANK:
                $sppPaymentMethod = PaymentMethod::swedbank();
                break;
            case BankLink::PAYMENT_METHOD_NORDEA:
                $sppPaymentMethod = PaymentMethod::nordea();
                break;
            case BankLink::PAYMENT_METHOD_SEB:
                $sppPaymentMethod = PaymentMethod::seb();
                break;
            case BankLink::PAYMENT_METHOD_DNB:
                $sppPaymentMethod = PaymentMethod::dnb();
                break;
            case BankLink::PAYMENT_METHOD_DANSKE:
                $sppPaymentMethod = PaymentMethod::danske();
                break;
            default:
                $sppPaymentMethod = null;
                break;
        }

        return $sppPaymentMethod;
    }

    /**
     * @param string $paymentMethod
     *
     * @return array
     */
    private function getServiceType($paymentMethod)
    {
        $bankLtEnabled = (bool) Configuration::get('SB_BANK_LT');
        $bankLvEnabled = (bool) Configuration::get('SB_BANK_LV');
        $bankEeEnabled = (bool) Configuration::get('SB_BANK_EE');

        $sppServiceName = $this->module->name;

        if ($bankLtEnabled && BankLink::PAYMENT_METHOD_SWEDBANK == $paymentMethod) {
            $sppServiceName = $this->module->l('Swedbank', self::FILENAME);
            $sppServiceType = ServiceType::litBank();
        } elseif ($bankLtEnabled && BankLink::PAYMENT_METHOD_SEB == $paymentMethod) {
            $sppServiceName = $this->module->l('Seb', self::FILENAME);
            $sppServiceType = ServiceType::sebLit();
        } elseif ($bankLtEnabled && BankLink::PAYMENT_METHOD_NORDEA == $paymentMethod) {
            $sppServiceName = $this->module->l('Nordea', self::FILENAME);
            $sppServiceType = ServiceType::nrdLit();
        } elseif ($bankLtEnabled && BankLink::PAYMENT_METHOD_DNB == $paymentMethod) {
            $sppServiceName = $this->module->l('Dnb', self::FILENAME);
            $sppServiceType = ServiceType::undefined();
        } elseif ($bankLtEnabled && BankLink::PAYMENT_METHOD_DANSKE == $paymentMethod) {
            $sppServiceName = $this->module->l('Danske', self::FILENAME);
            $sppServiceType = ServiceType::undefined();
        } elseif ($bankLvEnabled && BankLink::PAYMENT_METHOD_SWEDBANK == $paymentMethod) {
            $sppServiceName = $this->module->l('Swedbank', self::FILENAME);
            $sppServiceType = ServiceType::ltvBank();
        } elseif ($bankLvEnabled && BankLink::PAYMENT_METHOD_SEB == $paymentMethod) {
            $sppServiceName = $this->module->l('Seb', self::FILENAME);
            $sppServiceType = ServiceType::sebLtv();
        } elseif ($bankEeEnabled && BankLink::PAYMENT_METHOD_SWEDBANK == $paymentMethod) {
            $sppServiceName = $this->module->l('Swedbank', self::FILENAME);
            $sppServiceType = ServiceType::estBank();
        } elseif ($bankEeEnabled && BankLink::PAYMENT_METHOD_SEB == $paymentMethod) {
            $sppServiceName = $this->module->l('Seb', self::FILENAME);
            $sppServiceType = ServiceType::sebEst();
        } elseif ($bankEeEnabled && BankLink::PAYMENT_METHOD_NORDEA == $paymentMethod) {
            $sppServiceName = $this->module->l('Nordea', self::FILENAME);
            $sppServiceType = ServiceType::nrdEst();
        } else {
            $sppServiceType = null;
        }

        return [$sppServiceType, $sppServiceName];
    }
}
